import { ValueType, CollectedTypes } from '@previewjs/type-analyzer';
import ts from 'typescript';

type SerializableValue = SerializableArrayValue | SerializableBooleanValue | SerializableFunctionValue | SerializableMapValue | SerializableNodeValue | SerializableNullValue | SerializableNumberValue | SerializableObjectValue | SerializablePromiseValue | SerializableSetValue | SerializableStringValue | SerializableUndefinedValue | SerializableUnknownValue;
type SerializableArrayValue = {
    kind: "array";
    items: SerializableValue[];
};
declare function array(items: SerializableValue[]): SerializableArrayValue;
declare const EMPTY_ARRAY: SerializableArrayValue;
type SerializableBooleanValue = {
    kind: "boolean";
    value: boolean;
};
declare function boolean(value: boolean): SerializableBooleanValue;
declare const FALSE: SerializableBooleanValue;
declare const TRUE: SerializableBooleanValue;
type SerializableFunctionValue = {
    kind: "function";
    source: string;
};
declare function fn(source: string): SerializableFunctionValue;
type SerializableMapValue = {
    kind: "map";
    values: SerializableObjectValue;
};
declare function map(values: SerializableObjectValue): SerializableMapValue;
type SerializableNodeValue = {
    kind: "node";
    tag: string;
    props: SerializableObjectValue;
    children: SerializableValue[] | null;
};
declare function node(tag: string, props: SerializableObjectValue, children?: SerializableValue[] | null): SerializableNodeValue;
declare const NULL: {
    readonly kind: "null";
};
type SerializableNullValue = typeof NULL;
type SerializableNumberValue = {
    kind: "number";
    value: number;
};
declare function number(value: number): SerializableNumberValue;
type SerializableObjectValue = {
    kind: "object";
    entries: SerializableObjectValueEntry[];
};
type SerializableObjectValueEntry = {
    kind: "key";
    key: SerializableValue;
    value: SerializableValue;
} | {
    kind: "spread";
    value: SerializableValue;
};
declare function object(entries: SerializableObjectValueEntry[]): SerializableObjectValue;
type SerializablePromiseValue = {
    kind: "promise";
    value: {
        type: "reject";
        message: string | null;
    } | {
        type: "resolve";
        value: SerializableValue;
    };
};
declare function promise(value: {
    type: "reject";
    message: string | null;
} | {
    type: "resolve";
    value: SerializableValue;
}): SerializablePromiseValue;
declare const EMPTY_OBJECT: SerializableObjectValue;
declare const EMPTY_MAP: SerializableMapValue;
type SerializableSetValue = {
    kind: "set";
    values: SerializableArrayValue;
};
declare function set(values: SerializableArrayValue): SerializableSetValue;
declare const EMPTY_SET: SerializableSetValue;
type SerializableStringValue = {
    kind: "string";
    value: string;
};
declare function string(value: string): SerializableStringValue;
declare const UNDEFINED: {
    readonly kind: "undefined";
};
type SerializableUndefinedValue = typeof UNDEFINED;
type SerializableUnknownValue = {
    kind: "unknown";
    source?: string;
};
declare function unknown(source?: string): SerializableUnknownValue;
declare const UNKNOWN: SerializableUnknownValue;

/**
 * Generates a valid value for the given type.
 */
declare function generateSerializableValue(type: ValueType, collected: CollectedTypes, options?: {
    fieldName?: string;
    random?: boolean;
}): SerializableValue;

declare function parseSerializableValue(expression: ts.Expression): SerializableValue;

declare function serializableValueToJavaScript(value: SerializableValue): string;

export { EMPTY_ARRAY, EMPTY_MAP, EMPTY_OBJECT, EMPTY_SET, FALSE, NULL, SerializableArrayValue, SerializableBooleanValue, SerializableFunctionValue, SerializableMapValue, SerializableNodeValue, SerializableNullValue, SerializableNumberValue, SerializableObjectValue, SerializableObjectValueEntry, SerializablePromiseValue, SerializableSetValue, SerializableStringValue, SerializableUndefinedValue, SerializableUnknownValue, SerializableValue, TRUE, UNDEFINED, UNKNOWN, array, boolean, fn, generateSerializableValue, map, node, number, object, parseSerializableValue, promise, serializableValueToJavaScript, set, string, unknown };
